@@ -123,6 +123,7 @@ class ModelInfoBase(object):
         return self.as_string()
 
 class ModelInfo(ModelInfoBase):
+    """Automatic info creator for model classes, basing on objects"""
     class _Meta(ModelInfoBase._Meta):
         fieldsets = None
         row_template = '<tr><th>%s</th><td>%s</td></tr>'
@@ -185,7 +186,27 @@ class ModelInfo(ModelInfoBase):
 
         return mark_safe(u'\n'.join(ret))
 
+class ModelListItem(object):
+    model_list = None
+    _row_str = ''
+    obj = None
+
+    def __init__(self, model_list, row_str, obj):
+        self.model_list = model_list
+        self._row_str = row_str
+        self.obj = obj
+
+    def __unicode__(self):
+        return self.row_str
+
+    def get_row_str(self):
+        return self._row_str
+    row_str = property(get_row_str)
+
 class ModelList(ModelInfoBase):
+    """Automatic list creator for model classes, basing on QuerySets"""
+    item_class = ModelListItem
+
     class _Meta(ModelInfoBase._Meta):
         td_template = '<td class="%(field_name)s">%(value)s</td>'
         th_template = '<th>%s</th>'
@@ -286,25 +307,54 @@ class ModelList(ModelInfoBase):
                         group_rows.append(group_row)
 
             # Single row
+            row = self.render_single_object(obj)
+            ret = self.item_class(self, '\n'.join(group_rows) + row, obj)
 
+            """
             row = []
 
             for i, f_name in enumerate(fields):
                 f_value = self.get_field_display_value(f_name, obj)
-                f_value = self.get_linkable_field_value(obj, f_name, f_value, force=not self._meta.list_display_links and i == 0)
+                f_value = self.get_linkable_field_value(
+                        obj, f_name, f_value,
+                        force=not self._meta.list_display_links and i == 0,
+                        )
 
                 if self._meta.show_if_none or f_value is not None:
                     row.append(self._meta.td_template%{'field_name': f_name, 'value': f_value})
                 else:
                     row.append(self._meta.td_template%{'field_name': f_name, 'value': '&nbsp;'})
 
-            ret = ModelListItem(
-                    '\n'.join(group_rows) + self._meta.tr_template%(''.join(row) + self.render_buttons_cell(obj)),
-                    obj,
-                    )
+            ret = self.item_class(self, '\n'.join(group_rows) + self._meta.tr_template%(
+                        ''.join(row) + self.render_buttons_cell(obj)), obj)
+            """
             
             yield ret
 
+    def render_single_object(self, obj, tr_template=None, td_template=None):
+        # Customized templates
+        tr_template = tr_template or self._meta.tr_template
+        td_template = td_template or self._meta.td_template
+
+        fields = self._meta.fields or self.get_model_fields()
+        row = []
+
+        for i, f_name in enumerate(fields):
+            f_value = self.get_field_display_value(f_name, obj)
+            f_value = self.get_linkable_field_value(
+                    obj, f_name, f_value,
+                    force=not self._meta.list_display_links and i == 0,
+                    )
+
+            if self._meta.show_if_none or f_value is not None:
+                row.append(td_template%{'field_name': f_name, 'value': f_value})
+            else:
+                row.append(td_template%{'field_name': f_name, 'value': '&nbsp;'})
+
+        ret = tr_template%(''.join(row) + self.render_buttons_cell(obj))
+
+        return ret
+
     def as_string(self):
         fields = self._meta.fields or self.get_model_fields()
 
@@ -325,30 +375,3 @@ class ModelList(ModelInfoBase):
 
         return mark_safe(ret)
 
-    """
-    def render_single_object(self, obj):
-        fields = self._meta.fields or self.get_model_fields()
-        row = []
-
-        for i, f_name in enumerate(fields):
-            f_value = self.get_field_display_value(f_name, obj)
-            f_value = self.get_linkable_field_value(obj, f_name, f_value, force=not self._meta.list_display_links and i == 0)
-
-            if self._meta.show_if_none or f_value is not None:
-                row.append(self._meta.td_template%{'field_name': f_name, 'value': f_value})
-            else:
-                row.append(self._meta.td_template%'&nbsp;')
-
-        ret = '\n'.join(group_rows) + self._meta.tr_template%(''.join(row) + self.render_buttons_cell(obj))
-
-        return mark_safe(ret)
-    """
-
-class ModelListItem(object):
-    def __init__(self, row_str, obj):
-        self.row_str = row_str
-        self.obj = obj
-
-    def __unicode__(self):
-        return self.row_str
-
